#!/bin/bash -v
#
# Install/Uninstall script for KlamAV
#

yesno ()
{
  while true; do
  echo ""
  echo "$1 (Y/n) ? "
  read ans
  case X$ans in
   X|Xy|XY) return 0;;
   Xn|XN) return 1;;
  esac
  done
}
    
install() {
	echo "***************************************************************************************************"
	echo "                            Welcome to the KlamAV Installation Process"
	echo "***************************************************************************************************"
	echo ""
	echo "I'm going to attempt to install the following components:  "
	echo "1. The component that will give KlamAV real-time access to files. (dazuko: http://www.dazuko.org) "
	echo "2. The KlamAV front-end and the mail scanning plugin (klammail: part of the KlamAV project).. (http://www.sf/net/projects/klamav). "
	echo ""
	echo "Do you still want to continue?"
	if yesno; then
		continue
	else
		exit 1
	fi

	echo "***************************************************************************************************"
	echo "                            Checking your ClamAV Installation"
	echo "***************************************************************************************************"
	echo ""

	if which clamscan; then
		echo "Clamscan binary present..."
	else
		echo "Clamscan binary missing... is ClamAV really installed?"
		exit 1
	fi
	if which freshclam; then
		echo "freshclam binary present..."
	else
		echo "freshclam binary missing... is ClamAV really installed?"
		exit 1
	fi
	
	
	
	echo ""
	echo "Testing existence of 'clamav' user group.."
	echo ""

	if grep -q 'clamav' /etc/group; then 
		echo "Good: clamav group already exists."
		echo ""
	else
		echo "Group 'clamav' doesn't exist, so I'm creating it.."
		echo ""
		su -c 'groupadd clamav' || exit 1
	fi

	
	#Build dazuko...
	echo ""
	echo "***************************************************************"
	echo "Step One:"
	echo "Building Real-Time Scanning Component (dazuko kernel module)..."
	echo "Please note that this step is necessary even if you already have"
	echo "dazuko installed. At the moment, KlamAV needs to know the exact "
	echo "location of dazuko - so it installs it in /lib/modules/`uname -r`/ and loads"
	echo "it from there."
	echo "***************************************************************"
	echo ""
	cd src/$DAZUKO || exit 1
	./configure || exit 1
	make || exit 1
	echo "Need your root password "
	echo "	-to copy component to /lib/modules/`uname -r`..."
	echo "	-to insert component (dazuko kernel module)..."
	if [ -e "/dev/dazuko" ]; then
		echo "- device file /dev/dazuko already exists. I need to reinstall it to ensure the permissions are correct."
		su -c 'rm -f /dev/dazuko' || exit 1
		su -c 'cp dazuko.*o /lib/modules/`uname -r`;sleep 1;insmod /lib/modules/`uname -r`/dazuko.*o;sleep 1;rm -f /dev/dazuko;sleep 1;mknod -m 600 /dev/dazuko c `grep dazuko /proc/devices | sed "s/ .*//"` 0;sleep 1;chown $USER:clamav /dev/dazuko' || exit 1

	else
		su -c 'cp dazuko.*o /lib/modules/`uname -r`;sleep 1;insmod /lib/modules/`uname -r`/dazuko.*o;sleep 1;mknod -m 600 /dev/dazuko c `grep dazuko /proc/devices | sed "s/ .*//"` 0;sleep 1;chown $USER:clamav /dev/dazuko' || exit 1
	fi
	
	
	echo "Need your root password to set you up as the owner of the component (ownership will be user: $USER group: clamav)..."
	su -c 'chown $USER:clamav /dev/dazuko' || exit 1
	echo ""
	sleep 1
	echo "*************************************************************************"
	echo "Finished building Real-Time Scanning Component (dazuko kernel module)..."
	echo "*************************************************************************"
	echo ""
	
	#Build KlamAV...
	echo "***************************************************************"
	echo "Final Step:"
	echo "Building KlamAV Front-End the mail scanning plugin (klammail: part of the KlamAV project)..."
	echo "***************************************************************"
	echo ""
	cd ../..  || exit 1
	./configure  || exit 1
	make  || exit 1
	echo "Need your root password to install..."
	su -c 'make install'  || exit 1
}


uninstall() {


	echo "***************************************************************************************************"
	echo "                            Welcome to the KlamAV Un-Install Process"
	echo "***************************************************************************************************"
	echo ""
	echo "I'm going to attempt to uninstall the following components:  "
	echo "1. The component that will give KlamAV real-time access to files. (dazuko: http://www.dazuko.*org) "
	echo "2. The KlamAV front-end and the mail scanning plugin (klammail: part of the KlamAV project).. (http://www.sf/net/projects/klamav). "
	echo ""
	echo "Do you still want to continue?"
	if yesno; then
		continue
	else
		exit 1
	fi

	#Uninstall dazuko...
	echo ""
	echo "********************************************************************"
	echo "Step one:"
	echo "Uninstalling Real-Time Scanning Component (dazuko kernel module)..."
	echo "********************************************************************"
	echo ""
	cd src/$DAZUKO || exit 1
	make clean || exit 1
	echo "Need your root password to remove module from /proc/devices..."
	su -c 'rmmod dazuko'
	echo "Need your root password to remove component in /lib/modules/`uname -r`..."
	su -c 'rm -f /lib/modules/`uname -r`/dazuko.*o' || exit 1
	echo "Need your root password to remove component module's device in /dev (/dev/dazuko)..."
	su -c 'rm -f /dev/dazuko' || exit 1
	
	sleep 1
	echo "*************************************************************************"
	echo "Finished uninstalling Real-Time Scanning Component (dazuko kernel module)..."
	echo "*************************************************************************"
	echo ""
	
	#Uninstall KlamAV...
	echo "***************************************************************"
	echo "Final Step:"
	echo "Uninstalling KlamAV Front-End..."
	echo "***************************************************************"
	echo ""
	cd ../..  || exit 1
	make clean || exit 1
	echo "Need your root password to uninstall..."
	su -c 'make uninstall'  || exit 1
	sleep 1
	echo "*************************************************************************"
	echo "Finished uninstalling KlamAV Front-End..."
	echo "*************************************************************************"
	echo ""

}


# Execution starts here
if [ "$1" = "" ]; then
	echo ""
	echo "ERROR: You need to type either 'sh ./klamavsetup.sh install' or 'klamavsetup uninstall'."
	exit 1
fi
ACTION=$1
#echo "Action: $ACTION"

DAZUKO=dazuko-2.0.6
TMPPATH=`echo $PATH| cut -d ':' -f 1`

case $ACTION in
	install)
	install
	;;
	
	uninstall)
	uninstall
	;;

	*)
	echo "Usage: install { [install | uninstall] [clamdonly]} "
	exit 1
	;;
esac

