/*
 *
 *  Clamdmail.c is based loosely on clamdscan.c by Tomasz Kojm.
 *
 *  This program takes a mail message as input from stdin, uses rfc822.c to extract
 *  attachments to a temp directory, gets clamd to scan them, and
 *  handles the message appropriately.
 *
 *  Copyright (C) 2003 Robert Hogan <robert@roberthogan.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <pwd.h>
#include <clamav.h>

#include "options.h"
//#include "others.h"
//#include "shared.h"
#include "defaults.h"
#include "ripmime-api.h"
//#include "output.h"

#define CLAMDMAIL_VER "0.14"
#define BUFFSIZE 1024



/* this local macro takes care about freeing memory at exit */
/*
#define mexit(i)    if(opt) free_opt(opt);			    \
		    mprintf("*Memory freed. Exit code: %d\n", i);   \
		    exit(i);
*/
#define mexit(i)    exit(i)

struct s_info {
    int signs; /* number of signatures loaded */
    int dirs; /* number of scanned directories */
    int files; /* number of scanned files */
    int ifiles; /* number of infected files */
    int notremoved; /* number of not removed files (if --remove) */
    int notmoved; /* number of not moved files (if --move) */
    int errors; /*  ... of errors */
    long int blocks; /* number of read 16kb blocks */
};


void help(void);
void printtag(void);
int quarantine(int fd, char *quarname, long timestamp, char **virname, char *From, char *FromR, char *To, char *out, struct optstruct *opt);

struct s_info claminfo;
short printinfected = 0;
short int mprintf_stdout;

int clamdscan(struct optstruct *opt)
{
	int ds, dms, ret;
	double mb;
	struct timeval t1, t2;
	struct timezone tz;
	int fd, no;
	char *virname;
	const char *bndrystore;
	struct cl_node *root = NULL;
	struct cl_limits limits;
	const char *tmpdir;
	char *tmpfil;
	char *pfx;
	char *tmper;
	char *dir;
	struct passwd *user = NULL;
       	FILE *tmp;
       	FILE *fs;
        int bytes;
	long int size = 0;
    	char buff[BUFFSIZE];
	struct cl_node *trie = NULL;
	int threads = 0;
	struct RIPMIME_object *rmob;

	rmob=(struct RIPMIME_object*) mmalloc(sizeof(struct RIPMIME_object));

	ret = RIPMIME_init(rmob);




    if(optc(opt, 'V')) {
	mprintf("clamdmail "CLAMDMAIL_VER" \n");
	mexit(0);
    }

    if(optc(opt, 'h')) {
	free_opt(opt);
    	help();
    }

    if(optc(opt, 'i')) printinfected = 1;
    else printinfected = 0;



    memset(&claminfo, 0, sizeof(struct s_info));

    gettimeofday(&t1, &tz);
		/* njh@bandsman.co.uk: BeOS */
#if !defined(C_CYGWIN) && !defined(C_BEOS)
		if((user = getpwnam(UNPUSER)) == NULL) {
		    mprintf("@Can't get information about user %s\n", UNPUSER);
		    mexit(60); /* this is critical problem, so we just exit here */
		}
#endif

		if((tmpdir = getargl(opt, "tempdir")) == NULL)
			tmpdir = getenv("TMPDIR");

		if(tmpdir == NULL)
#ifdef P_tmpdir
			tmpdir = P_tmpdir;
#else
			tmpdir = "/tmp";
#endif


		if(checkaccess(tmpdir, UNPUSER, W_OK) != 1) {
			mprintf("@Can't write to the temporary directory.\n");
			mexit(64);
		}
		/* generate the temporary directory */

		dir = cl_gentemp(tmpdir);
		if(mkdir(dir, 0700)) {
			mprintf("@Can't create the temporary directory %s\n", dir);
			mexit(63); /* critical */
		}

		if(user)
			chown(dir, user->pw_uid, user->pw_gid);


			tmper = tempnam(tmpfil, pfx);
			/*if((tmp = tmpfile()) == NULL) {
        			cli_dbgmsg("Can't generate tmpfile().\n");
        			return CL_ETMPFILE;
    			}
			*/

    			fd = open(tmper,O_RDWR|O_CREAT, S_IRWXU);

    			while((bytes = read(0, buff, BUFFSIZE)) > 0) {

			        if(write(fd, buff, bytes) != bytes) {
           			 	cli_dbgmsg("clamdmail -> Can't write() file.\n");
            				close(fd);
            				return CL_EMEM;
        			}
    			}

			if(fsync(fd) == -1) {
			        cli_dbgmsg("fsync() failed for descriptor %d\n", fd);
        			close(fd);
        			return CL_EFSYNC;
    			}
			/*if (!(tmp = fdopen(fd,"r"))){
        			mprintf("@Can't open file \n");
        			return 54;
        		}
    			lseek(fd, 0, SEEK_SET);*/
			/*close(fd);*/
		/*
		 * Extract the attachments into the temporary directory
		 */
		ret = RIPMIME_decode(rmob, tmper, dir);
		if(ret == 0) {
			/* pfix permissions of extracted files */
			fixperms(dir);

			ret = client(dir, opt, &virname);
		}else
			ret = CL_CLEAN;

		/* remove the directory - as clamav */
		clamav_rmdirs(dir);

		free(dir);

		ret = spoolstdin(fd, ret, &virname, &bndrystore, opt);
		
		unlink(tmper);




    mexit(ret);
}

int spoolstdin(int fd, int ret, char **virname, const char **bndrystore, struct optstruct *opt)
{
	int bytes;
	int i, j;
	long int size = 0;
	char buff[BUFFSIZE];
	char string[1000];
	char To[1000];
	char ReplyTo[1000];
	char DelivTo[1000];
	char From[1000];
	char ToR[1000];
	char FromR[1000];
	char boundary[1000];
	char host[256];  /* could be HOST_NAME_MAX+1 on POSIX 1003.1-2001 */
	struct timeval tv;
	struct timezone tz;
	char *s;
	char *storage;
	FILE *tmp;
	int pcount = 0;
	time_t starttime;
	struct tm *tm;
	char *out = NULL;
	const char *format;
	size_t out_length = 0;
	time_t when;
	

	strcpy (To, "To:");
	strcpy (From, "From:");
	strcpy (ReplyTo, "Reply-To:");
	strcpy (DelivTo, "Delivered");

	gethostname(host, sizeof(host));
	gettimeofday(&tv, &tz);
	time(&when);
	tm = localtime(&when);
	format = "%a, %_d %b %Y %H:%M:%S %z";
	do {
		out_length += 200;
		out = (char *) realloc (out, out_length);
		out[0] = '\1';
	} while (strftime (out, out_length, format, tm) == 0 && out[0] != '\0');

	if(ret == CL_VIRUS) {
		lseek(fd, 0, SEEK_SET);
		if (!(tmp = fdopen(fd,"r"))){
			mprintf("@Can't open file \n");
			return 54;
		}

    		mprintf_stdout = 1;

		while (fgets(string, sizeof(string), tmp)) {
			if (strstr(string, To) && !(strstr(string, ReplyTo)) && !(strstr(string,DelivTo))){
				strcpy(To, string);
				strcpy(ToR, string);
			}
			if (strstr(string, From)){
				strcpy(FromR, string);
			}
			if (strncmp(string, "\n", 1) == 0 && pcount != 0){
				break;
			}
			++pcount;
		}

		if (strcmp(To,"To:") == 0){
			strcat(To, " ");
			strcat(To, getargl(opt, "admin"));
			strcat(To, "\n");
		}
		strcat(From, " ");
		strcat(From, getargl(opt, "admin"));
		strcat(From, "\n");
		strcat(ReplyTo, " ");
		strcat(ReplyTo, getargl(opt, "admin"));
		strcat(ReplyTo, "\n");
		mprintf("%s", From);
		mprintf("%s", ReplyTo);
		mprintf("%s", To);
		mprintf("Subject: Virus %s found in attached mail by ClamAV.\n", *virname);
		mprintf("Date: %s\n", out);
		if(!(optl(opt, "quar"))){
			mprintf("MIME-Version: 1.0\n");
			mprintf("Content-Type: multipart/mixed;\n");
			mprintf("  boundary=\"----------=_%d\"\n",tv.tv_sec);
			mprintf("X-Virus-Scan: Scanned by clamdmail %s on %s (found %s);\n\t%s\n", 
				CLAMDMAIL_VER, host, *virname, out);
			mprintf("\n");
			mprintf("\n");
			mprintf("------------=_%d\n",tv.tv_sec);
		}
		mprintf("Content-Type: text/plain;\n");
		mprintf("  charset=\"us-ascii\"\n");
		mprintf("Content-Transfer-Encoding: quoted-printable\n");
		if((optl(opt, "quar"))){
			mprintf("X-Virus-Scan: Scanned by clamdmail %s on %s (found %s);\n\t%s\n", 
				CLAMDMAIL_VER, host, *virname, out);
			mprintf("\n");
			mprintf("ClamAV anti-virus scanner has intercepted and quarantined a message addressed to you.\n");
		}
		else
		{
			mprintf("\n");
			mprintf("ClamAV anti-virus scanner has detected a virus in the attached message.\n");
		}
		mprintf("\n");
		mprintf("The following is a summary of the infected message:\n");
		mprintf("\n");
		mprintf("Virus name: %s\n", *virname);
		mprintf("%s", FromR);
		mprintf("%s", ToR);
		mprintf("\n");
		mprintf("Please be aware that a virus spread by email normally forges the \n");
		mprintf("address of the sender. There is a good chance that the infected message\n");
		mprintf("was not received from the sender listed above. \n");
		mprintf("\n");
		if(!(optl(opt, "quar"))){
			mprintf("------------=_%d\n",tv.tv_sec);
			mprintf("Content-Type: message/rfc822; x-virus-type=original\n");
			mprintf("Content-Description: Infected Message - Open At Your Own Risk\n");
			mprintf("Content-Disposition: inline\n");
			mprintf("Content-Transfer-Encoding: 8bit\n");
			mprintf("\n");
			fflush(stdout);

			lseek(fd, 0, SEEK_SET);
			while((bytes = read(fd, buff, BUFFSIZE)) > 0) {

				if(write(1, buff, bytes) != bytes) {
				    cli_dbgmsg("clamdmail -> Can't write() file.\n");
					close(fd);
					return CL_EMEM;
				}
			 }
			mprintf("------------=_%d--\n",tv.tv_sec);
		}
		fflush(stdout);

		if(optl(opt, "quar") && ret==CL_VIRUS){
			char *quarname;
			quarname = getargl(opt, "quar");
			quarantine(fd, quarname, tv.tv_sec, virname, From, FromR, To, out, opt);
		}
		fclose(tmp);

	} else if(ret == CL_CLEAN) {
		int bcnt = 0;
	    	lseek(fd, 0, SEEK_SET);
		if (!(tmp = fdopen(fd,"r"))){
			mprintf("@Can't open file %s\n", fd);
			return 54;
		}

    		mprintf_stdout = 1;

		if ((*bndrystore) && (optl(opt, "tag"))){	
			while (fgets(string, sizeof(string), tmp)) {
				if (strstr(string, *bndrystore)){
					++bcnt;
					if (bcnt == 3){
						printtag();
					}
				}
				fputs(string, stdout);
			}
		}else{
			/* Insert Virus Scanner Header here */
			/* Ultimately, this should include hostname, datestamp and version 
			 * of clamd (from connection to clamd, not as compiled!), possibly
			 * as a Received: header?
			 * X-Virus-Scan: Scanned by clamdmail 0.12/clamd 0.60 on host.domain.com (no viruses); 25 Aug 2003 12:34:56 -0000
			 */
			while (fgets(string, sizeof(string), tmp)) {
				if (*string == '\n') {
					mprintf("X-Virus-Scan: Scanned by clamdmail %s on %s (no viruses);\n\t%s\n\n", 
						CLAMDMAIL_VER, host, out);
					break;
				}
				fputs(string, stdout);
			}
			while (fgets(string, sizeof(string), tmp)) {
				fputs(string, stdout);
			}
		}

		if(optl(opt, "tag"))
			if (!(*bndrystore))
				printtag();

		fclose(tmp);
	} else
		if(!printinfected)
		    mprintf("stdin: %s\n", cl_perror(ret));
	
	return ret;
}

void printtag(void)
{

    mprintf_stdout = 1;
    mprintf("\n");
    mprintf("----------------------------------------------------------------------------\n");
    mprintf("			This message was scanned by\n");
    mprintf("		ClamAV Open Source Anti-Virus Technology\n");
    mprintf("\n");
    mprintf("			http://clamav.elektrapro.com\n");
    mprintf("----------------------------------------------------------------------------\n");
    mprintf("\n");
    fflush(stdout);

}

void help(void)
{

    mprintf_stdout = 1;

    mprintf("\n");
    mprintf("		   ClamAV Daemon Mail Processing Client "CLAMDMAIL_VER"\n");
    mprintf("		   (c) 2003 Robert Hogan <robert@roberthogan.net>\n");
    mprintf("		   	Uses a lot of code written by:\n");
    mprintf("		       Tomasz Kojm <zolw@konarski.edu.pl>\n");
    mprintf("		       Nigel Horne <njh@bandsman.co.uk>\n");
    mprintf("	  \n");
    mprintf("    --help		-h	Show help\n");
    mprintf("    --version		-V	Print version number and exit\n");
    mprintf("    --spam			Pass message to spamc/spamd first.\n");
    mprintf("    --tag			Tag messages as scanned.\n");
    mprintf("    --admin=you@you.com		The from address you want clamdmail to use when informing recipients\n");
    mprintf("				they have been sent an infected message. Basically, your postmaster account.\n");
    mprintf("				If you use --quar, you will get notified of infections at this address.\n");
    mprintf("    --quar=/your/quarantine	Dump infected mail in this directory and inform recipient and postmaster.\n");
    mprintf("    --mta=/path/to/your/mta	The path to your MTA's equivalent of qmail-inject. The hardcoded default\n");
    mprintf("                           	is /var/qmail/bin/qmail-inject, so if qmail-inject is somehwere else or\n");
    mprintf("                           	you use something else for injecting  mails, then you need to put it in here.\n");
    mprintf("                           	This info is needed for sending notifications to the admin.\n");


    exit(0);
}

int quarantine(int fd, char *quarname, long timestamp, char **virname, char *From, char *FromR, char *To, char *out, struct optstruct *opt)
{
		char *filename;
		char buff[BUFFSIZE];
		int bytes;
		int errno;
		int fda;
		FILE *fs;
	    int pfds[2];
	    int childpid;

                filename = cli_malloc(strlen(quarname) + 8);
		snprintf(filename, "%s/%iXXXXXX", quarname, timestamp);


                /*
                 * TODO: add a HAS_MKSTEMP property
                 */
#if     defined(C_LINUX) || defined(C_BSD) || defined(HAVE_MKSTEMP)
                fda = mkstemp(filename);
#else
                (void)mktemp(filename);
                fda = open(filename, O_WRONLY|O_CREAT|O_EXCL, 0600);
#endif


                if(fda < 0) {
                        cli_errmsg("%s: %s\n", filename, strerror(errno));
                        return 0;
                }

		lseek(fd, 0, SEEK_SET);

		while((bytes = read(fd, buff, BUFFSIZE)) > 0) {
	
			if(write(fda, buff, bytes) != bytes) {
			    cli_dbgmsg("clamdmail -> Can't write() file.\n");
			    close(fd);
			    return CL_EMEM;
			}
		}
		close(fda);



	    if ( pipe(pfds) == -1 ) {
		perror("Failed to create pipe; quitting\n");
		exit(1);
	    }

	    if ( ( childpid=fork() ) == -1 ) {
		perror("Failed to fork; quitting\n");
		exit(2);
	    }

	    if ( childpid == 0 ) {
		close(0);       /* close normal stdout */
		dup(pfds[0]);   /* make stdout same as pfds[1] */
		close(pfds[1]); /* we don't need this */
		if(optl(opt, "mta")){
	     		char *mta;
	     		mta = getargl(opt, "mta");
			execlp(mta, mta, NULL);
		}else{
			execlp("/var/qmail/bin/qmail-inject", "/var/qmail/bin/qmail-inject", NULL);
		}
	    } else {
	    	close(1);       /* close normal stdin */
		dup(pfds[1]);   /* make stdin same as pfds[0] */
		close(pfds[0]);
		mprintf("From: %s\n", getargl(opt,"admin"));
		mprintf("To: %s\n", getargl(opt, "admin"));
		mprintf("Subject: Virus %s Found In Mail %s By ClamAV.\n", *virname, To);
		mprintf("Date: %s\n", out);
		mprintf("Content-Type: text/plain;\n");
		mprintf("  charset=\"us-ascii\"\n");
		mprintf("Content-Transfer-Encoding: quoted-printable\n");
		mprintf("\n");
		mprintf("ClamAV has intercepted and quarantined a message with the following info:\n");
		mprintf("\n");
		mprintf("%s", FromR);
		mprintf("%s", To);
		mprintf("\n");
		mprintf("Quarantined as: %s", filename);
		mprintf("\n");
		mprintf("\n");
		fflush(stdout);
	    }	

		
}
